@@ -1,5 +1,5 @@
 package Dancer2::Plugin::MarkdownFilesToHTML ;
-$Dancer2::Plugin::MarkdownFilesToHTML::VERSION = '0.009';
+$Dancer2::Plugin::MarkdownFilesToHTML::VERSION = '0.010';
 use 5.010; use strict; use warnings;
 
 use Carp;
@@ -91,11 +91,9 @@ sub _set_options {
   my %options = ();
   foreach my $option (keys %defaults) {
     if ($path) {
-      $options{$option} = $route->{$path}{$option}
-                        // $config->{defaults}{$option}
-                        // $defaults{$option};
+      $options{$option} = $route->{$path}{$option} // $config->{defaults}{$option} // $defaults{$option};
     } else {
-      $options{$option} = $config->{$option} // $defaults{$option};
+      $options{$option} = $route->{$option} // $config->{defaults}{$option} // $defaults{$option};
     }
   }
 
@@ -106,6 +104,19 @@ sub _set_options {
   return \%options;
 }
 
+sub _get_options {
+  my $s = shift;
+  my $options = shift;
+  my $resource = shift;
+
+  $options = _set_options($options, '', $s->config);
+  my $is_abs = File::Spec->file_name_is_absolute($resource);
+  if (!$is_abs) {
+    $resource = File::Spec->catfile($options->{file_root}, $resource);
+  }
+  return ($options, $resource);
+}
+
 # Gathers lists of files in a directory and sends them off to mdfile_2html for
 # parsing or cache retrieval
 sub mdfiles_2html {
@@ -115,7 +126,7 @@ sub mdfiles_2html {
 
   # If options haven't been set yet, get defaults from cnofig file
   if (!$options->{set}) {
-    $options = _set_options($options, '', $s->config);
+    ($options, $dir) = $s->_get_options($options, $dir);
   }
 
   my $html = '';
@@ -137,6 +148,7 @@ sub mdfiles_2html {
       @files = @matching_files;
     }
   }
+
   if ($options->{exclude_files}) {
     foreach my $excluded_file (@{$options->{exclude_files}}) {
       @files = grep { $_ ne $excluded_file } @files;
@@ -149,7 +161,7 @@ sub mdfiles_2html {
     $html                      .= $file_html;
     $toc                       .= $file_toc;
   }
-  return ($html, $toc);
+  return wantarray ?  ($html, $toc) : $html;
 }
 
 # Sends the markdown file to get parsed or retrieves html version from cache,
@@ -157,23 +169,23 @@ sub mdfiles_2html {
 sub mdfile_2html {
 	my ($s, $file, $options) = @_;
 
-  # If options haven't been set yet, get defaults from config file
+  # If options haven't been set yet, get defaults from cnofig file
   if (!$options->{set}) {
-    $options = _set_options($options, '', $s->config);
+    ($options, $file) = $s->_get_options($options, $file);
   }
 
-  # generate the cache if it doesn't exist
-  if (!-d 'lib/data/markdown_files/cache' && $ENV{DANCER_ENVIRONMENT} ne 'testing') {
-    make_path 'lib/data/markdown_files'
+  # generate the cache directory if it doesn't exist
+  my $cache_dir = File::Spec->catfile(dirname($options->{'file_root'}), 'md_file_cache');
+  if (!-d $cache_dir) {
+    make_path $cache_dir or die "Cannot make cache directory $!";
   }
 
   # generate unique cache file name appended with values of two options
   my $cache_file = $file;
   $cache_file =~ s/\///g;
-  my $cache_dir = $ENV{DANCER_ENVIRONMENT} eq 'testing' ? 'xt' : 'lib';
-  $cache_file = "$cache_dir/data/markdown_files/cache/$cache_file"
-                . $options->{linkable_headers}
-                . $options->{generate_toc};
+  $cache_file = File::Spec->catfile(
+                 $cache_dir,
+                 $cache_file . $options->{linkable_headers} . $options->{generate_toc});
 
   # check for cache hit
   if (-f $cache_file && $options->{cache}) {
@@ -197,7 +209,8 @@ sub mdfile_2html {
   # See if we can cache and return the output without further processing
   # generate_toc makes linkable_headers true so we just need to test linkable_headers option
   if (!$options->{linkable_headers}) {
-    return $s->_cache_data($options, $cache_file, $file, $out);
+    my ($html, $toc) = $s->_cache_data($options, $cache_file, $file, $out);
+    return wantarray ?  ($html, $toc) : $html;
   }
 
   my $tree     = HTML::TreeBuilder->new_from_content($out);
@@ -259,7 +272,7 @@ Dancer2::Plugin::MarkdownFilesToHTML
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 SYNOPSIS
 
